unit fmBC;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms;

type

  TBaseComponent = class(TComponent)
  protected
    FHandle : THandle;
    FEnabled : boolean;
    FDesignActive : boolean;
    procedure WMQueryEndSession(var Message : TMessage); message WM_QUERYENDSESSION;
    procedure WndProc(var Message : TMessage); virtual;
    procedure SetEnabled(AEnabled : boolean); virtual;
    procedure DoSetEnabled(AEnabled : boolean); virtual;
    procedure Loaded; override;
    property Handle : THandle read FHandle;
  public
    property Enabled : boolean read FEnabled write SetEnabled default false;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  end;

implementation

 { TBaseComponent }

constructor TBaseComponent.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FHandle := 0;
  FEnabled := false;
end;

destructor TBaseComponent.Destroy;
begin
  Enabled := false;
  inherited Destroy;
end;

procedure TBaseComponent.Loaded;
begin
  inherited;
  if Enabled then
  begin
    FEnabled := false;
    SetEnabled(true);
  end;
end;

procedure TBaseComponent.WndProc(var Message : TMessage);
begin
  if Message.Msg = WM_QUERYENDSESSION then
    Message.Result := 1
  else
  try
    Dispatch(Message);
  except
    Application.HandleException(Self);
  end;
end;

procedure TBaseComponent.WMQueryEndSession(var Message : TMessage);
begin
  try Enabled := false;
  except;
  end;
  inherited;
  Message.Result := integer(true);
end;

procedure TBaseComponent.SetEnabled(AEnabled : boolean);
begin
  try
    if (FDesignActive or (not (csDesigning in ComponentState))) and
      (not (csLoading in ComponentState)) and (AEnabled <> FEnabled) then DoSetEnabled(AEnabled);
    FEnabled := AEnabled;
  except
    raise;
  end;
end;

procedure TBaseComponent.DoSetEnabled(AEnabled : boolean);
begin
  if AEnabled then
    FHandle := AllocateHwnd(WndProc)
  else
  begin
    if FHandle <> 0 then DeallocateHwnd(FHandle);
    FHandle := 0;
  end;
end;

end.
