unit fmClipMon;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,fmbc;
{$R FMCLIPMON.DCR}
type
  TfmClipboardMonitor = class(TBaseComponent)
  private
    FNextViewer : THandle;
    FOnChange : TNotifyEvent;
    FDataFormats : TStrings;
    function GetDataFormats : TStrings;
    procedure WMChangeCBChain(var Message : TMessage); message WM_CHANGECBCHAIN;
    procedure WMDrawClipboard(var Message : TMessage); message WM_DRAWCLIPBOARD;
  protected
    procedure DoSetEnabled(AEnabled : boolean); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function GetDataString(FormatName : string) : string;
    property DataFormats : TStrings read GetDataFormats;
  published
    property Enabled;
    property OnChange : TNotifyEvent read FOnChange write FOnChange;
  end;
function GetFormatName(AFormat : integer) : string;
function GetFormatIndex(FormatName : string) : integer;
function HasFormat(Handle : THandle; Index : integer) : boolean;

implementation

 { TfmClipboardMonitor }

constructor TfmClipboardMonitor.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FDataFormats := TStringList.Create;
end;

destructor TfmClipboardMonitor.Destroy;
begin
  //don't swap these lines!
  inherited Destroy;
  FDataFormats.Free;
end;

procedure TfmClipboardMonitor.DoSetEnabled(AEnabled : boolean);
begin
  if AEnabled then
  begin
    inherited DoSetEnabled(true);
    FNextViewer := SetClipboardViewer(Handle);
  end
  else
  begin
    ChangeClipboardChain(Handle, FNextViewer);
    inherited DoSetEnabled(false);
  end;
end;

procedure TfmClipboardMonitor.WMChangeCBChain(var Message : TMessage);
begin
  with Message do
  begin
    if THandle(WParam) = FNextViewer then
      FNextViewer := LParam
    else if FNextViewer <> 0 then
      SendMessage(FNextViewer, WM_CHANGECBCHAIN, WParam, LParam);
    Result := 0;
  end;
end;

procedure TfmClipboardMonitor.WMDrawClipboard(var Message : TMessage);
begin
  if Assigned(FOnChange) then FOnChange(self);
  if FNextViewer <> 0 then SendMessage(FNextViewer, WM_DRAWCLIPBOARD, 0, 0);
end;

function TfmClipboardMonitor.GetDataFormats : TStrings;
var
  Fmt : integer;
begin
  Result := FDataFormats;
  Result.Clear;

  if not OpenClipboard(Handle) then Exit;
  try
    Fmt := EnumClipboardFormats(0);
    while Fmt <> 0 do
    begin
      FDataFormats.Add(GetFormatName(Fmt));
      Fmt := EnumClipboardFormats(Fmt);
    end;
  finally
    CloseClipboard;
  end;
end;

function TfmClipboardMonitor.GetDataString(FormatName : string) : string;
var
  H : THandle;
  Data : PChar;
  Len : integer;
begin
  if DataFormats.IndexOf(FormatName) = -1 then raise Exception.Create('Format not available');

  OpenClipboard(Handle);
  try
    H := GetClipboardData(GetFormatIndex(FormatName));
    Data := GlobalLock(H);
    Len := GlobalSize(H);
    SetLength(Result, Len);
    Move(Data^, Result[1], Len);
    GlobalUnlock(H);
  finally
    CloseClipboard;
  end;
end;

function HasFormat(Handle : THandle; Index : integer) : boolean;
var
  Fmt : integer;
begin
  result := false;
  if not OpenClipboard(Handle) then Exit;
  try
    Fmt := EnumClipboardFormats(0);
    while Fmt <> 0 do
    begin
      if Fmt = Index then
      begin
        result := true;
        break;
      end;
      Fmt := EnumClipboardFormats(Fmt);
    end;
  finally
    CloseClipboard;
  end;
end;

function GetFormatName(AFormat : integer) : string;
const
  FormatNames : array[1..16] of string =
    ('TEXT', 'BITMAP', 'METAFILEPICT', 'SYLK', 'DIF', 'TIFF',
    'OEMTEXT', 'DIB', 'PALETTE', 'PENDATA', 'RIFF', 'WAVE',
    'UNICODETEXT', 'ENHMETAFILE', 'HDROP', 'LOCALE');
begin
  if (AFormat >= CF_TEXT) and (AFormat <= CF_LOCALE) then
    Result := FormatNames[AFormat]
  else
  begin
    SetLength(Result, 128);
    SetLength(Result, GetClipboardFormatName(AFormat, PChar(Result), 128));
  end;
end;

function GetFormatIndex(FormatName : string) : integer;
const
  FormatNames = ':TEXT:BITMAP:METAFILEPICT:SYLK:DIF:TIFF:OEMTEXT:DIB:PALETTE:' +
    'PENDATA:RIFF:WAVE:UNICODETEXT:ENHMETAFILE:HDROP:LOCALE:';
var
  S : string;
begin
  s := UpperCase(FormatName);
  case Pos(':' + S + ':', FormatNames) of
    1 : Result := CF_TEXT;
    6 : Result := CF_BITMAP;
    13 : Result := CF_METAFILEPICT;
    26 : Result := CF_SYLK;
    31 : Result := CF_DIF;
    35 : Result := CF_TIFF;
    40 : Result := CF_OEMTEXT;
    48 : Result := CF_DIB;
    52 : Result := CF_PALETTE;
    60 : Result := CF_PENDATA;
    68 : Result := CF_RIFF;
    73 : Result := CF_WAVE;
    78 : Result := CF_UNICODETEXT;
    90 : Result := CF_ENHMETAFILE;
    102 : Result := CF_HDROP;
    108 : Result := CF_LOCALE;
  else
    Result := RegisterClipboardFormat(PChar(S));
  end;
end;


end.
