{******************************************************
 Fulgore's Misc Components
 Freeware. Copyright(c) 2000 by Eli Yukelzon
 If you want to use this code, please first email me at:
                  eli@yukelzon.com
 Component Info:
 Exception Logger - 
 Just a little something that will probably simplify the path
 of geting feedback from your users... Just customize the look of the
 form and Enjoy...
 ******************************************************}

unit fmELog;

interface


uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs , EFrm;

type
  TfmLogInfoEvent = procedure (Sender : TObject ; var ExtraInfo : TStringList) of object ;

  TfmELogger = class(TComponent)
  private
     FExtraLogInfo : TfmLogInfoEvent ;
     FLogDir : string ;
     FLogFile : string ;
     FExitOnError : boolean ;
  public
     constructor Create(AOwner : TComponent) ; override ;
     procedure GlobalHandler(Sender: TObject; E: Exception);
  published
     property ExtraLogInfo : TfmLogInfoEvent read FExtraLogInfo
                                             write FExtraLogInfo ;
     property LogDir : string read FLogDir write FLogDir ;
     property LogFile : string read FLogFile write FLogFile ;
     property ExitOnError : boolean read FExitOnError
                                 write FExitOnError default False ;
  end;

implementation

constructor TfmELogger.Create(AOwner : TComponent) ;
begin
     inherited Create(AOwner) ;
     FLogFile := 'errors.log' ;
     if not (csDesigning in ComponentState) then
        Application.OnException := GlobalHandler ;
end ;

procedure TfmELogger.GlobalHandler(Sender: TObject; E: Exception);
var
   f : TErrorForm ;
   UserNotes : TStringList ;
   LogFile : TStringList;
   ExtraInfo : TStringList ;
   sFileName : string;
begin
   f := TErrorForm.Create(nil) ;
   UserNotes := TStringList.Create ;
   f.ErrClassLabel.Caption := 'Exception Class: ' + E.ClassName;
   f.ErrMsgLabel.Caption := E.Message;
   try
      f.ShowModal ;
      UserNotes.Assign( f.UserMemo.Lines ) ;
   finally
      f.Release ;
   end ;
   if FLogDir = '' then FLogDir := ExtractFilePath(ParamStr(0)) ;
   sFileName := FLogDir + FLogFile ;
   LogFile := TStringList.Create;
   try
      if FileExists(sFileName) then LogFile.LoadFromFile(sFileName);
      LogFile.Add( 'Application Error at ' + FormatDateTime('hh:nn am/pm', Now)
         + ' on ' + FormatDateTime('mm/dd/yy', Now));
      LogFile.Add('Exception Class: ' + E.ClassName);
      LogFile.Add('Exception Message: ' + E.Message);
      if UserNotes.Count > 0 then begin
         LogFile.Add('') ;
         LogFile.Add('User Notes:');
         LogFile.AddStrings(UserNotes);
      end ;
      UserNotes.Free ;
      if Assigned(FExtraLogInfo) then begin
         ExtraInfo := TStringList.Create ;
         FExtraLogInfo(self, ExtraInfo) ;
         if ExtraInfo.Count > 0 then
            LogFile.AddStrings(ExtraInfo) ;
         ExtraInfo.Free ;
      end ;
      LogFile.Add(StringOfChar('=',79)) ;
      LogFile.SaveToFile(sFileName);
   finally
      LogFile.Free;
   end;
   if FExitOnError then Application.Terminate ;
end ;

end.
