unit fmOneInst;

{ Application single instance monitor }

interface

uses
  Windows,
  SysUtils,
  Messages,
  Classes,
  Forms,fmbc;
{$R onewp.res}

type
  TInstanceRunEvent = procedure(Sender : TObject; Parameters : TStrings) of object;

type
  PfmMemMapArr = ^TfmMemMapArr;
  TfmMemMapArr = record
    hPrevInst, hPrevAppWin, hPrevMainWin, hMonWin : THandle;
  end;

  TExistsEvent = procedure(Sender : TObject;
    hPrevInst : THandle; hPrevAppWin, hPrevMainWin : HWND; var Switch : boolean) of object;

  TfmOneInstance = class(TbaseComponent)
  private
    FMapHandle : THandle;
    FPrevInst,
    FPrevAppWin,
    FPrevMainWin : THandle;

    FView      : PfmMemMapArr;
    FMapName   : string;
    FNoAutoTerminate : boolean;

    FOnExists : TExistsEvent;
    FOnInstanceRun : TInstanceRunEvent;

    procedure WMCopyData(var Msg : TMessage); message WM_COPYDATA;
  protected
    { Protected declarations }
    procedure TriggerExistsEvent(hPrevInst, hPrevAppWin, hPrevMainWin : THandle; var Switch : boolean); virtual;
    procedure TriggerInstanceRunEvent(Parameters : TStrings); virtual;
    procedure SetEnabled(AEnabled : boolean);
    procedure DoSetEnabled(AEnabled : boolean);
    procedure CreateMapping;
  public
    { Public declarations }
    destructor Destroy; override;
    property PrevInstance : THandle read FPrevInst;
    property FPrevMainWindow : THandle read FPrevMainWin;
    property FPrevAppWindow : THandle read FPrevAppWin;
  published
    property MapName : string read FMapName write FMapName;
    property NoAutoTerminate : boolean read FNoAutoTerminate write FNoAutoTerminate;
    property OnExists : TExistsEvent read FOnExists write FOnExists;
    property OnInstanceRun : TInstanceRunEvent read FOnInstanceRun write FOnInstanceRun;
    property Enabled;
  end; { TfmOneInstance }

implementation

const
  OneInstAlreadyExists = 'Another instance of the application is started';

procedure TfmOneInstance.TriggerExistsEvent(hPrevInst, hPrevAppWin, hPrevMainWin : THandle; var Switch : boolean);
begin
  if (assigned(FOnExists)) then
    FOnExists(Self, hPrevInst, hPrevAppWin, hPrevMainWin, Switch);
end; { TriggerExistsEvent }

procedure TfmOneInstance.WMCopyData(var Msg : TMessage); { private }
var
  ds : PCOPYDATASTRUCT;
  ParamsList : TStringList;
  Params : string;
begin
  with Msg do
  begin
    ds := PCOPYDATASTRUCT(LParam);
    params := StrPas(PChar(ds.lpData));
    ParamsList := TStringList.Create;
    ParamsList.Text := params;
    TriggerInstanceRunEvent(ParamsList);
    ParamsList.Free;
    Result := 1458;
  end; { with }
end; { WMCopyData }

procedure TfmOneInstance.TriggerInstanceRunEvent(Parameters : TStrings);
begin
  if (assigned(FOnInstanceRun)) then FOnInstanceRun(Self, Parameters);
end; { TriggerInstanceRunEvent }

procedure TfmOneInstance.SetEnabled(AEnabled : boolean);
begin
  if (FMapName = '') and AEnabled then
    raise Exception.Create('Memory mapping name should be specified to enable Instance Counter.');
  inherited;
end;

procedure TfmOneInstance.DoSetEnabled(AEnabled : boolean);
begin
  inherited;
  if AEnabled then
    CreateMapping
  else if FMapHandle <> 0 then
  begin
    CloseHandle(FMapHandle);
    FMapHandle := 0;
  end;
end;

procedure TfmOneInstance.CreateMapping;
var
  b, s : boolean;
  ds : TCOPYDATASTRUCT;
  ParamsList : TStringList;
  i : integer;
  Params : string;
  ParentForm : TComponent;
begin
  if not (csDesigning in ComponentState) then
  begin
    FMapHandle := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0, sizeof(TfmMemMapArr), PChar(FMapName));
    if FMapHandle <> 0 then
    begin
      b := GetLastError = ERROR_ALREADY_EXISTS;
      FView := nil;
      try
        FView := MapViewOfFile(FMapHandle, FILE_MAP_ALL_ACCESS, 0, 0, sizeof(TfmMemMapArr));
        if FView = nil then raise Exception.Create('Failed to initialize ElOneInstance object');
        if b then
        begin
          s := false;
          TriggerExistsEvent(FView.HPrevInst, FView.hPrevAppWin, FView.hPrevMainWin, s);
          FPrevInst := FView.HPrevInst;
          FPrevMainWin := FView.hPrevMainWin;
          FPrevAppWin := FView.hPrevAppWin;
          if s and (not NoAutoTerminate) then
          begin
            ParamsList := TStringList.Create;
            for i := 0 to ParamCount do
              ParamsList.Add(ParamStr(i));
            Params := ParamsList.Text;
            ParamsList.Free;
            ds.dwData := 0;
            ds.cbData := length(params) + 1;
            ds.lpData := PChar(Params);
            SendMessage(FView.hMonWin, WM_COPYDATA, Handle, integer(@ds));
            if FView.hPrevMainWin <> 0 then
              SetForegroundWindow(FView.hPrevMainWin)
            else
              SetForegroundWindow(FView.hPrevAppWin);
            Application.Terminate;
          end;
          if NoAutoTerminate then
          begin
            UnmapViewOfFile(FView);
            FView := nil;
            CloseHandle(FMapHandle);
            raise Exception.Create(OneInstAlreadyExists);
          end;
        end
        else
        begin
          FView.hPrevInst := HInstance;
          FView.hPrevAppWin := Application.Handle;
          if Application.MainForm <> nil then
            FView.hPrevMainWin := Application.MainForm.Handle
          else
          begin
            ParentForm := self;
            while not (ParentForm is TForm) and (ParentForm.Owner <> nil) do
              ParentForm := ParentForm.Owner;
            if ParentForm <> nil then
              FView.hPrevMainWin := TForm(ParentForm).Handle
            else
              FView.hPrevMainWin := 0;
          end;
          FView.hMonWin := Handle;
        end;
      finally
        UnmapViewOfFile(FView);
      end;
    end
    else
      raise Exception.Create('Failed to initialize fmOneInstance object');
  end;
end;

destructor TfmOneInstance.Destroy;
begin
  if FMapHandle <> 0 then CloseHandle(FMapHandle);
  FMapHandle := 0;
  inherited;
end; { Destroy }

end.
