{******************************************************
 Fulgore's Misc Components
 Freeware. Copyright(c) 2000 by Eli Yukelzon
 If you want to use this code, please first email me at:
                  eli@yukelzon.com
 Component Info:
 What & When -
 If you need to create some timed events, then this
 component is for you. Just fill the collections properties
 and that's all...
 ******************************************************}
unit fmWW;

interface

uses
  Windows,Messages, SysUtils, Classes, Graphics,ComCtrls,
  Controls, Forms, Dialogs,ExtCtrls,dsgnintf,fmWWDlg ;
{$R wwlog.res}
type
  TfmDay = (dSunday, dMonday, dTuesday, dWednesday, dThursday, dFriday, dSaturday) ;
  TfmDays = set of TfmDay ;

  TfmDate = (d1st, d2nd, d3rd, d4th, d5th, d6th, d7th, d8th, d9th, d10th, d11th,
             d12th, d13th, d14th, d15th, d16th, d17th, d18th, d19th, d20th, d21st,
             d22nd, d23rd, d24th, d25th, d26th, d27th, d28th, d29th, d30th, d31st,
             dLastDayOfMonth) ;
  TfmDates = set of TfmDate ;


  TfmWWEvent = class(TCollectionItem)
  private
     FEnabled : boolean ;
     FTheDate : string ;
     FDays : TfmDays ;
     FDates : TfmDates ;
     FTheTime : string ;
     FOnTimedEvent : TNotifyEvent ;
     procedure SetTime(s : string) ;
     procedure SetDate(s : string) ;
  public
     constructor Create(Collection : TCollection) ; override ;
  published
     property Enabled : boolean read FEnabled write FEnabled default True ;
     property Dates : TfmDates read FDates write FDates default [] ;
     property Days : TfmDays read FDays write FDays default [] ;
     property OnTimedEvent : TNotifyEvent read FOnTimedEvent write FOnTimedEvent ;
     property TheDate : string read FTheDate write SetDate ;
     property TheTime : string read FTheTime write SetTime ;
  end ;

  TfmWW = class;
  TItemClass = class of TfmWWEvent ;

  TfmWWEvents = class(TCollection)
  private
     FOwner : TfmWW ;
     function GetEvent(i : integer) : TfmWWEvent ;
     procedure SetEvent(i : integer; e : TfmWWEvent) ;
  protected
     function GetOwner : TPersistent ; override ;
  public
     constructor Create(AOwner : TfmWW; ItemClass : TItemClass);
     property Items[Index: Integer]: TfmWWEvent read GetEvent write SetEvent ; default ;
  end ;


  // primary component declaration
  TfmWW = class(TComponent)
  private
     FTimer : TTimer ;
     FEvents : TfmWWEvents ;
     function GetActive : boolean ;
     procedure CheckQueue(Sender : TObject) ;
     function LastDayOfMonth : boolean ;
     procedure SetActive(b : boolean) ;
  protected
     procedure Loaded ; override ;
  public
     property Active : boolean read GetActive write SetActive ;
     constructor Create(AOwner : TComponent) ; override ;
     destructor Destroy ; override ;
  published
     property Events : TfmWWEvents read FEvents write FEvents ;
  end;

  TfmDateEditor = class(TStringProperty)
     function GetAttributes : TPropertyAttributes ; override ;
     procedure Edit ; override ;
  end ;

  TfmTimeEditor = class(TStringProperty)
     function GetAttributes : TPropertyAttributes ; override ;
     procedure Edit ; override ;
  end ;



implementation

constructor TfmWWEvent.Create(Collection: TCollection);
begin
     inherited ;
     FEnabled := True ;
end ;

procedure TfmWWEvent.SetTime(s : string) ;
var
   d : TDateTime ;
begin
     try
        d := StrToTime(s) ;
        FTheTime := FormatDateTime(LongTimeFormat, d) ;
     except
        MessageDlg('Invalid time format', mtError, [mbOK], 0) ;
     end ;
end ;

procedure TfmWWEvent.SetDate(s : string) ;
var
   d : TDateTime ;
begin
     if s = '' then
        FTheDate := s
     else
        try
           d := StrToDate(s) ;
           FTheDate := FormatDateTime(ShortDateFormat, d) ;
        except
           MessageDlg('Invalid date format', mtError, [mbOK], 0) ;
        end ;
end ;

constructor TfmWWEvents.Create(AOwner : TfmWW; ItemClass : TItemClass);
begin
     inherited Create(ItemClass) ;
     FOwner := AOwner ;
end ;

function TfmWWEvents.GetOwner : TPersistent ;
begin
     Result := FOwner ;
end ;

function TfmWWEvents.GetEvent(i : integer): TfmWWEvent ;
begin
     Result := TfmWWEvent(inherited Items[i]);
end;

procedure TfmWWEvents.SetEvent(i : integer; e : TfmWWEvent) ;
begin
     Items[i].Assign(e);
end;


constructor TfmWW.Create(AOwner : TComponent) ;
begin
     inherited ;
     FEvents := TfmWWEvents.Create(self, TfmWWEvent) ;
     FTimer := TTimer.Create(self) ;
     FTimer.Enabled := False ;
     FTimer.OnTimer := CheckQueue ;
end ;

destructor TfmWW.Destroy ;
begin
     FTimer.Free ;
     FEvents.Clear ;
     FEvents.Free ;
     inherited ;
end ;

procedure TfmWW.Loaded ;
begin
     inherited ;
     Active := not (csDesigning in ComponentState) ;
end ;

function TfmWW.GetActive : boolean ;
begin
     Result := FTimer.Enabled ;
end ;

procedure TfmWW.SetActive(b : boolean) ;
begin
     FTimer.Enabled := b ;
end ;

procedure TfmWW.CheckQueue(Sender : TObject) ;
var
   DayOfMonth : word ;
   dummy1, dummy2 : word ;
   CurrEvent : integer ;
begin
     DecodeDate(Date, dummy1, dummy2, DayOfMonth) ;
     for CurrEvent := 0 to FEvents.Count - 1 do
        with TfmWWEvent( FEvents.Items[CurrEvent] ) do
           if FEnabled then

           if (FTheTime = TimeToStr(Time)) and
              ((FTheDate = DateToStr(Date)) or
               (TfmDate(DayOfMonth - 1) in FDates) or
               (TfmDay(DayOfWeek(Date)-1) in FDays) or
               (LastDayOfMonth and (dLastDayOfMonth in FDates) ) ) then
              if Assigned(FOnTimedEvent) then FOnTimedEvent(self) ;

end ;

function TfmWW.LastDayOfMonth : boolean ;
var
   Year, Month, Day : word ;
   NumberOfDays : integer ;
const
    DaysInMonth: array[1..12] of Integer =
                 (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
     DecodeDate(Date, Year, Month, Day) ;
     NumberOfDays := DaysInMonth[Month];
     if (Month = 2) and IsLeapYear(Year) then
        Inc(NumberOfDays) ;
     Result := (Day = NumberOfDays) ;
end;

//prop ed

function TfmDateEditor.GetAttributes : TPropertyAttributes ;
begin
     Result := inherited GetAttributes + [paDialog] ;
end ;

procedure TfmDateEditor.Edit ;
begin
     with TfrmDateTimeEditor.Create(nil) do
        try
           DateTimePicker.Kind := dtkDate ;
           if GetStrValue <> '' then
              DateTimePicker.Date := StrToDate(GetStrValue)
           else
              DateTimePicker.Date := Date ;
           if ShowModal = mrOK then
              SetStrValue( DateToStr(DateTimePicker.Date) ) ;
        finally
           Release ;
        end ;
end ;

function TfmTimeEditor.GetAttributes : TPropertyAttributes ;
begin
     Result := inherited GetAttributes + [paDialog] ;
end ;

procedure TfmTimeEditor.Edit ;
begin
     with TfrmDateTimeEditor.Create(nil) do
        try
           DateTimePicker.Kind := dtkTime ;
           if GetStrValue <> '' then
              DateTimePicker.Time := StrToTime(GetStrValue)
           else
              DateTimePicker.Time := Time ;
           if ShowModal = mrOK then
              SetStrValue( TimeToStr(DateTimePicker.Time) ) ;
        finally
           Release ;
        end ;
end ;


end.
